//----------------------------------------------------------------------------
//*****************************************************************************
// File Name            : msg_buf.v
//-----------------------------------------------------------------------------
// Function             : message buffer and arbitartion 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2010/5/31
// -.-- 2010/11/12
//*****************************************************************************
module msg_buf
( CLK,
  RESET_N,
  A_DAT_EN,
  A_DAT,
  A_N_FULL,
  B_DAT_EN,
  B_DAT,
  B_N_FULL,
  O_DAT,
  O_DAT_EN,
  O_BUSY,
  MON,
); 

parameter data_width = 8   ;
parameter data_no = data_width -1;
parameter adr_width = 8   ;
parameter adr_no = adr_width -1;
parameter mem_size = (1 << adr_width)-1;
   
   
input                   CLK;
input                   RESET_N;
input                   A_DAT_EN;
input     [data_no:0]   A_DAT;
output                  A_N_FULL;
input                   B_DAT_EN;
input     [data_no:0]   B_DAT;
output                  B_N_FULL;
output                   O_DAT_EN;
output     [data_no:0]   O_DAT;
input                   O_BUSY;
output                  MON;


wire                  REN_A;
wire                  RDAT_EN_A;
wire    [data_no:0]   RDAT_A;
wire            FULL_A;
wire            N_FULL_A;
wire            EMPTY_A;

wire                  REN_B;
wire                  RDAT_EN_B_TMP;
reg                  RDAT_EN_B;
wire    [data_no:0]   RDAT_B;
wire            FULL_B;
wire            N_FULL_B;
wire            EMPTY_B;

wire    A_REQ;
wire    A_REDAY;
reg     B_REQ;
wire    B_REDAY;
wire                   ABT_DAT_EN;
wire     [data_no:0]   ABT_DAT;   


reg                  O_DAT_EN;
reg            REN_O;
wire            FULL_O;
wire            N_FULL_O;
wire            EMPTY_O;
   
reg                  RDAT_EN_A_DFF;
reg    [data_no:0]   RDAT_A_DFF;
reg                  A_DAT_EN_DFF;
reg                  ABT_DAT_EN_DFF;
reg    [data_no:0]   O_DAT_DFF;
reg                  O_DAT_EN_DFF;
reg                  O_BUSY_DFF;
   
fifo
#(data_width,
  data_no,
  adr_width,
  adr_no,
  mem_size
)

DUT_A(
.CLK    (CLK),
.RESET_N  (RESET_N), 
.WDAT_EN    (A_DAT_EN),   
.WDAT   (A_DAT),  
.REN    (REN_A),     
.RDAT   (RDAT_A),
.RDAT_EN   (RDAT_EN_A),
.EMPTY  (EMPTY_A),
.FULL   (FULL_A),
.N_FULL   (N_FULL_A)
);

fifo
#(data_width,
  data_no,
  adr_width,
  adr_no,
  mem_size
)

DUT_B(
.CLK    (CLK),
.RESET_N  (RESET_N), 
.WDAT_EN    (B_DAT_EN),   
.WDAT   (B_DAT),  
.REN    (REN_B),     
.RDAT   (RDAT_B),
.RDAT_EN(RDAT_EN_B_TMP),
.EMPTY  (EMPTY_B),
.FULL   (FULL_B),
.N_FULL   (N_FULL_B)
);

assign A_REQ = ~EMPTY_A;
assign REN_A = A_REDAY;
assign A_N_FULL = N_FULL_A;
   
//assign B_REQ = ~EMPTY_B;

always @(posedge CLK or negedge RESET_N) begin
  if (RESET_N==1'b0)begin
      B_REQ <= 1'b0;
    end
  else begin
      B_REQ <= ~EMPTY_B ;
    end
end


   
assign REN_B = B_REDAY;
assign B_N_FULL = N_FULL_B;

assign O_REDAY = ~N_FULL_O;
   
   
abt
#(data_width,
  data_no
)

ABT(
.CLK    (CLK),
.RESET_N  (RESET_N), 
.A_REQ   (A_REQ),   
.A_REDAY (A_REDAY),   
.A_DAT_EN(RDAT_EN_A),  
.A_DAT   (RDAT_A),  
.B_REQ   (B_REQ),   
.B_REDAY (B_REDAY),   
.B_DAT_EN(RDAT_EN_B),  
.B_DAT   (RDAT_B),  
.O_REQ   (O_REQ),   
.O_REDAY (O_REDAY),   
.O_DAT_EN(ABT_DAT_EN),  
.O_DAT   (ABT_DAT)
);
   

fifo
#(data_width,
  data_no,
  adr_width,
  adr_no,
  mem_size
)

DUT_O(
.CLK    (CLK),
.RESET_N  (RESET_N), 
.WDAT_EN    (ABT_DAT_EN),   
.WDAT   (ABT_DAT),  
.REN    (REN_O),     
.RDAT   (O_DAT),
.RDAT_EN(O_DAT_EN_TMP),
.EMPTY  (EMPTY_O),
.FULL   (FULL_O),
.N_FULL (N_FULL_O)
);
   
    
always @(posedge CLK or negedge RESET_N) begin
  if (RESET_N==1'b0)begin
      RDAT_EN_B <= 1'b0;
    end
  else begin
      RDAT_EN_B <= RDAT_EN_B_TMP ;
    end
end


always @(posedge CLK or negedge RESET_N) begin
  if (RESET_N==1'b0)begin
      O_DAT_EN <= 1'b0;
    end
  else begin
      O_DAT_EN <= O_DAT_EN_TMP ;
    end
end

always @(posedge CLK or negedge RESET_N) begin
  if (RESET_N==1'b0)begin
      O_BUSY_DFF <= 1'b0;
    end
  else begin
       O_BUSY_DFF <= O_BUSY ;
    end
end


always @(posedge CLK or negedge RESET_N) begin
  if (RESET_N==1'b0)begin
      REN_O <= 1'b1;
    end
  else begin
     if ((REN_O==1'b1)&&(EMPTY_O==1'b0))
       REN_O <= 1'b0 ;
     else if ((O_BUSY==1'b0)&&(O_BUSY_DFF==1'b1))
       REN_O <= 1'b1;
     else
       REN_O <= REN_O; 
    end
end


always @(posedge CLK or negedge RESET_N) begin
  if (RESET_N==1'b0)begin
      A_DAT_EN_DFF <= 0;
      RDAT_EN_A_DFF <= 1'b0;
      RDAT_A_DFF <= 0;
      ABT_DAT_EN_DFF <= 0;
      O_DAT_EN_DFF <= 1'b0;
      O_DAT_DFF <= 0;
    end
  else begin
      A_DAT_EN_DFF <= A_DAT_EN ;
      RDAT_EN_A_DFF <= RDAT_EN_A ;
      RDAT_A_DFF <= RDAT_A;
      ABT_DAT_EN_DFF <= ABT_DAT_EN ;
      O_DAT_EN_DFF <= O_DAT_EN_TMP ;
      O_DAT_DFF <= O_DAT;
    end
end

   assign MON = (^RDAT_A_DFF)^RDAT_EN_A_DFF^A_DAT_EN_DFF^ABT_DAT_EN_DFF^(^O_DAT_DFF)^O_DAT_EN_DFF;

   
endmodule
